@@ -319,9 +319,9 @@ class View(AbstractCommand):
 
 
     options:
-        -b, --binning=INT[bp|kb|Mb|Gb]   Subsampling factor or fix value in
-                                         basepairs to use for binning
-                                         [default: 1].
+        -b, --binning=INT[bp|kb|Mb|Gb]   Rebin the matrix. If no unit is given, bins will
+                                         be merged by groups of INT. If a unit is given,
+                                         bins of that size will be generated. [default: 1]
         -c, --cmap=CMAP                  The name of a matplotlib colormap to
                                          use for the matrix. [default: Reds]
         -C, --circular                   Use if the genome is circular.
@@ -1167,9 +1167,11 @@ class Distancelaw(AbstractCommand):
                                             logbins. These slices will be in basepairs unit.
                                             Default is 1.1.
         -c, --centromeres=FILE              Positions of the centromeres separated by
-                                            a space and in the same order than the 
-                                            chromosomes. Discordant with the circular
-                                            option.
+                                            a space and in the same order as the 
+                                            chromosomes. This allows to plot chromosomal arms
+                                            separately. Note this will only work with --pairs
+                                            input, as the distance law needs to be recomputed.
+                                            Incompatible with the circular option.
         -C, --circular                      Enable if the genome is circular. Discordant 
                                             with the centromeres option.
         -d, --dist-tbl=FILE1[,FILE2,...]    Directory to the file or files containing the 
@@ -1304,7 +1306,7 @@ class Distancelaw(AbstractCommand):
             xs = xs[0]
             ps = ps[0]
         # Normalize and make the derivative
-        ps = hcdl.normalize_distance_law(xs, ps, inf)
+        ps = hcdl.normalize_distance_law(xs, ps, inf, arm_sup)
         # Gave new names for the different samples.
         if self.args["--labels"]:
             labels = self.args["--labels"]
